name = "pynif"

from .phrase import NIFPhrase
from .context import NIFContext
from .collection import NIFCollection
from .prefixes import NIFPrefixes

__all__ = [ 'NIFPhrase', 'NIFContext', 'NIFCollection', 'NIFPrefixes' ]
